import logging, os, zc.buildout
import utils

class Mkdir:

    def __init__(self, buildout, name, options):
        self.name, self.options = name, options
        options['path'] = os.path.join(
                              buildout['buildout']['directory'],
                              options['path'],
                              )
        if not os.path.isdir(os.path.dirname(options['path'])):
            logging.getLogger(self.name).error(
                'Cannot create %s. %s is not a directory.',
                options['path'], os.path.dirname(options['path']))
            raise zc.buildout.UserError('Invalid Path')


    def install(self):
        path = self.options['path']
        logging.getLogger(self.name).info(
            'Creating directory %s', os.path.basename(path))
        os.mkdir(path)
        return path

    def update(self):
        pass


class ExportObject:

    def __init__(self, buildout, name, options):
        self.name, self.options = name, options
        instance = options['instance']
        self.obj_path = options['obj'].split()
        self.instance = buildout[instance]
        self.path = os.path.join(buildout['buildout']['directory'], 'parts', self.name)

    def install(self):
        br = utils.get_browser(self.instance)
        for obj1 in self.obj_path:
            if not utils.export_one(self.instance, obj1, br):
                raise zc.buildout.UserError('%s Export failed' % obj1)
        if not os.path.exists(self.path): os.mkdir(self.path)
        return self.path

    def update(self):
        pass

class CopyFile:

    def __init__(self, buildout, name, options):
        self.buildout, self.name, self.options = buildout, name, options
        self.path = os.path.join(buildout['buildout']['directory'], 'parts', self.name)
        self.files = options['files'].split()
        self.source = buildout[options['source']]
        self.target = buildout[options['target']]
        self.source_dir = options['source_dir']
        self.target_dir = options['target_dir']

    def install(self):
        """ copy file, create external method and run it """
        for f1 in self.files:
            utils.copy_file(self.source, self.source_dir, f1, self.target, self.target_dir)
        if not os.path.exists(self.path): os.mkdir(self.path)
        return self.path

    def update(self):
        pass

class ExternalMethod:

    def __init__(self, buildout, name, options):
        self.buildout, self.name, self.options = buildout, name, options
        file_path = os.path.join(buildout['buildout']['directory'], options['source'])
        self.source_dir, self.source_file = os.path.split(file_path)
        instance = options['instance']
        self.instance = buildout[instance]
        self.function = options['func']
        self.out_file = options['output']
        self.path = os.path.join(buildout['buildout']['directory'], 'parts', self.name)
        self.args = {}
        if 'args' in options:
            for arg in options['args'].split():
                self.args[arg] = options[arg]

    def install(self):
        """ copy file, create external method and run it """
        local_instance = {}
        local_instance['host'] = 'localhost'
        utils.copy_file(local_instance, self.source_dir, self.source_file,
                        self.instance, self.instance['extensions'])
        br = utils.get_browser(self.instance)
        utils.create_ext_method(self.instance, self.function, self.source_file, self.function, br)
        utils.run_ext_method(self.instance, self.function, self.args, self.out_file, br)
        if not os.path.exists(self.path): os.mkdir(self.path)
        return self.path
 

    def update(self):
        pass


class DelObject:

    def __init__(self, buildout, name, options):
        self.buildout, self.name, self.options = buildout, name, options
        instance = options['instance']
        self.obj = options['obj']
        self.host_name = buildout[instance]['host']
        self.port = buildout[instance]['port']
        self.zmi_user = buildout[instance]['zmi_user']
        self.zmi_pwd = buildout[instance]['zmi_pwd']
        self.plone_root = buildout[instance]['root']

    def install(self):
        """ copy file, create external method and run it """
        br = utils.get_browser(self.host_name, self.port, self.zmi_user, self.zmi_pwd)
        utils.del_object(self.host_name, self.port, self.plone_root, self.obj, br)
        path = os.path.join(buildout['buildout']['directory'], self.name)
        if not os.path.exists(path): os.mkdir(path)
        return path

    def update(self):
        pass



class ImportObject:

    def __init__(self, buildout, name, options):
        self.name, self.options = name, options
        instance = options['instance']
        self.obj = options['obj'].split()
        self.instance = buildout[instance]
        self.path = os.path.join(buildout['buildout']['directory'], 'parts', self.name)

    def install(self):
        """ copy file, create external method and run it """
        br = utils.get_browser(self.instance)
        for obj1 in self.obj:
            utils.import_one(self.instance, obj1, br)
        if not os.path.exists(self.path): os.mkdir(self.path)
        return self.path

    def update(self):
        pass





"""Do the migration
"""
import logging, os, sys, shutil


logger = logging.getLogger('migrator')


def loglevel():
    """Return DEBUG when -v is specified, INFO otherwise"""
    if len(sys.argv) > 1:
        if '-v' in sys.argv:
            return logging.DEBUG
    return logging.INFO


def main():
    logging.basicConfig(level=loglevel(),
                        format="%(levelname)s: %(message)s")
    logger.info('Starting migraton.')
    cur_dir = os.getcwd()
    mig_dir = os.path.join(cur_dir, 'migrator')
    if os.path.exists(mig_dir):
        logger.info('directory migrator already present')
        return
    os.mkdir(mig_dir)
    src_dir = os.path.dirname(__file__)
    scripts_dir = os.path.join(src_dir, 'scripts')
    mig_scripts_dir = os.path.join(mig_dir, 'scripts')
    shutil.copytree(scripts_dir, mig_scripts_dir)
    templates_dir = os.path.join(src_dir, 'templates')
    for f1 in os.listdir(templates_dir):
        src_file = os.path.join(templates_dir, f1)
        shutil.copy(src_file, mig_dir)
    os.chdir(mig_dir)
    os.system('buildout bootstrap')
    os.system('bin/buildout')



import os
import time
import base64
from zope.testbrowser.browser import Browser
from urllib2 import HTTPError

RETRIES = 1

Browsers = {}


def import_one(instance, obj, br):
    host_name, host_port, plone_root = (instance['host'], instance['port'], instance['root'])
    del_object(host_name, host_port, plone_root, obj, br)
    pos = obj.rfind('/')
    if pos == -1:
        url = 'http://%s:%s/%s/manage_importObject?file=%s.zexp&set_owner:int=0'% (host_name,
                   host_port, plone_root, obj)
    else:
        top_obj = obj[:pos]
        obj = obj[pos+1:]
        url = 'http://%s:%s/%s/%s/manage_importObject?file=%s.zexp&set_owner:int=0'% (host_name,
                   host_port, plone_root, top_obj, obj)

    for r1 in range(RETRIES):
        try:
            br.open(url)
        except HTTPError:
            # print '1=', sys.exc_info()[1]
            time.sleep(10)
            continue
        else:
            break
    s1 = br.contents
    return s1.find('successfully imported') != -1

def export_one(instance, obj, br):
    host_name, host_port, plone_root, user, export_folder = (instance['host'], instance['port'],
        instance['root'], instance['user'], instance['export'])
    pos = obj.rfind('/')
    if pos == -1:
        url = 'http://%s:%s/%s/manage_exportObject?id=%s&download:int=0&submit=Export'% (host_name,
                   host_port, plone_root, obj)
    else:
        top_obj = obj[:pos]
        obj = obj[pos+1:]
        url = 'http://%s:%s/%s/%s/manage_exportObject?id=%s&download:int=0&submit=Export'% (host_name,
                   host_port, plone_root, top_obj, obj)

    export_file = '%s/%s.zexp' % (export_folder, obj)

    cmd = 'rm -f %s' % export_file
    if host_name != 'localhost':
        cmd = 'ssh %s@%s %s' % (user, host_name, cmd)
    os.system(cmd)

    for r1 in range(RETRIES):
        try:
            f1 = br.open(url)
        except HTTPError:
            # print '1=', sys.exc_info()[1]
            time.sleep(10)
            continue
        else:
            break
    s1 = br.contents
    str1 = "successfully exported"
    return s1.find(str1) >= 0

def copy_file(source, source_dir, file_name, target, target_dir):
    if source['host'] == 'localhost':
        source_file = '%s/%s' % (source_dir, file_name)
    else:
        source_file = '%s@%s:%s/%s' % (source['user'], source['host'], source_dir, file_name)
    if target['host'] == 'localhost':
        target_file = target_dir
    else:
        target_file = '%s@%s:%s' % (target['user'], target['host'], target_dir)
    pos1 = source_file.find(':')
    pos2 = target_file.find(':')
    if pos1 == -1 and pos2 == -1:
        cmd = "cp %s %s" % (source_file, target_file)
    else:
        cmd = "scp %s %s" % (source_file, target_file)
    os.system(cmd)

def get_browser(instance):
    host_name, host_port, zmi_user, zmi_pwd = (instance['host'], instance['port'],
                                              instance['zmi_user'], instance['zmi_pwd'])
    key = '%s:%s' % (host_name, host_port)
    if key not in Browsers:
        br = Browser()
        br.addHeader('Authorization', 'Basic %s' % base64.encodestring('%s:%s' % (zmi_user, zmi_pwd)))
        Browsers[key] = br
    return Browsers[key]
    

def create_ext_method(instance, meth_id, module, function, br):
    host_name, host_port, plone_root = (instance['host'], instance['port'], instance['root'])
    url = 'http://%s:%s/%s/manage_delObjects?ids=%s' % (host_name, host_port, plone_root, meth_id)
    try:
        br.open(url)
    except HTTPError:
        pass
    url = 'http://%s:%s/%s/manage_addProduct/ExternalMethod/methodAdd' % (host_name, host_port, plone_root)
    br.open(url)
    br.getControl(name='id').value = meth_id
    br.getControl(name='module').value = module
    br.getControl(name='function').value = function
    br.getControl(name='submit').click()

def run_ext_method(instance, meth_id, args, out_file, br):
    host_name, host_port, plone_root = (instance['host'], instance['port'], instance['root'])
    url = 'http://%s:%s/%s/%s' % (host_name, host_port, plone_root, meth_id)
    if args:
        url_add = []
        for k,v in args.items():
            url_add.append('%s=%s' % (k, v))
        url_add = '&'.join(url_add)
        url = url + '?' + url_add
    br.open(url)
    fp = open(out_file, "w")
    fp.write(br.contents)
    fp.close()

def del_object(host_name, host_port, plone_root, obj, br):
    pos = obj.rfind('/')
    if pos == -1:
        url = 'http://%s:%s/%s/manage_delObjects?ids=%s' % (host_name, host_port, plone_root, obj)
    else:
        top_obj = obj[:pos]
        obj = obj[pos+1:]
        url = 'http://%s:%s/%s/%s/manage_delObjects?ids=%s' % (host_name, host_port, plone_root, top_obj, obj)
    try:
        br.open(url)
    except HTTPError:
        pass




from StringIO import StringIO

def dump_props(self, obj=None):
    out = StringIO()
    ptool = self
    if obj:
        obj = obj.split('__')
        for o1 in obj:
            ptool = getattr(ptool, o1)
    props = ptool.propertyIds()
    for p in props:
        typ, val = ptool.getPropertyType(p), ptool.getProperty(p)
        if typ == 'text':
            val = val.replace('\n', '\\n')
        print >> out, "%s, %s, %s" % (p, typ, val)
    return out.getvalue()

def load_props(self, obj=None):
    out = StringIO()
    ptool = self
    if obj:
        obj = obj.split('__')
        for o1 in obj:
            ptool = getattr(ptool, o1)
    fp = open('%s/Extensions/props.txt' % INSTANCE_HOME)
    prop_vals = {}
    for line in fp:
        prop, prop_type, val = line.split(', ', 2)
        val = val.strip()
        if not ptool.hasProperty(prop):
            print >> out, 'No such property: %s' % prop
            continue
        old_val = ptool.getProperty(prop)
        if prop_type in ('int', 'lines', 'boolean'):
            val = eval(val)
        elif prop_type == 'text':
            val = val.replace('\\n', '\n')
        elif prop_type != 'string':
            print >> out, 'Unknown type %s' % prop_type
            continue
        if val != old_val:
            prop_vals[prop] = val
            print >> out, 'Changed %s from %s to %s' % (prop, old_val, val)
    if prop_vals:
        ptool.manage_changeProperties(prop_vals)
    print >> out, 'Done'
    return out.getvalue()



from StringIO import StringIO
def load_memberdata(self):
    out = StringIO()
    mtool = self.portal_membership
    dtool = self.portal_memberdata
    props = dtool.propertyIds()
    fp = open('%s/Extensions/memberdata.txt' % INSTANCE_HOME)
    last_id = ''
    last_props = {}
    for line in fp:
        mem_id, prop, val = line.strip().split(',', 2)
        if mem_id != last_id:
            if last_id:
                mem = mtool.getMemberById(last_id)
                if not mem:
                    print >> out, 'mem %s does not exist' % last_id
                    continue
                if last_props:
                    mem.setMemberProperties(last_props)
                    last_props = {}
            last_id = mem_id
        val = val.replace('\\n', '\n')
        if val.startswith('(') and val.endswith(')'):
            val = eval(val)
        last_props[prop] = val
    print >> out, 'Done'
    return out.getvalue()

def dump_memberdata(self):
    out = StringIO()
    mtool = self.portal_membership
    dtool = self.portal_memberdata
    props = dtool.propertyIds()
    defaults = {}
    defaults['listed'] = ('1', 'True', 'on')
    defaults['login_time'] = ('2000/01/01',)
    defaults['last_login_time'] = ('2000/01/01',)
    defaults['wysiwyg_editor'] = ('Kupu',)
    defaults['error_log_update'] = ('0.0',)
    defaults['must_change_password'] = ('0', 'False')
    defaults['visible_ids'] = ('0', 'False')
    defaults['ext_editor'] = ('0', 'False')
    defaults['news_subscribed'] = ('0', 'False')
    defaults['user_categories'] = ('()', "('',)")

    for mem_id in mtool.listMemberIds():
        mem = mtool.getMemberById(mem_id)
        for prop in props:
            val = str(mem.getProperty(prop))
            if val == '':
                continue
            if prop in defaults.keys() and val in defaults[prop]:
                continue
            val = val.replace('\n', '\\n')
            print >> out, "%s,%s,%s" % (mem_id, prop, val)
    return out.getvalue()



from StringIO import StringIO
from Products.PlonePAS.Extensions.Install import activatePluginInterfaces
def meth(self):
    out = StringIO()
    activatePluginInterfaces(self, 'source_users')
    activatePluginInterfaces(self, 'source_groups')
    activatePluginInterfaces(self, 'portal_role_manager')
    print >> out, 'Done'
    return out.getvalue()



from setuptools import setup, find_packages
import os

version = '1.1'

entry_points="""
[console_scripts]
migrator = collective.migrator.migrator:main

[zc.buildout]
mkdir      = collective.migrator.recipes.migrator:Mkdir
copy_file  = collective.migrator.recipes.migrator:CopyFile
export_obj = collective.migrator.recipes.migrator:ExportObject
import_obj = collective.migrator.recipes.migrator:ImportObject
del_object = collective.migrator.recipes.migrator:DelObject
ext_method = collective.migrator.recipes.migrator:ExternalMethod
"""

setup(name='collective.migrator',
      version=version,
      description="Tool and buildout recipes for zope/plone content migration",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        "Framework :: Buildout",
        ],
      keywords='buildout recipe zope plone content migration',
      author='Suresh V',
      author_email='suresh@grafware.com',
      url='http://svn.plone.org/svn/collective/collective.migrator',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'zc.buildout',
          'zope.testbrowser',
      ],
      entry_points=entry_points,
      )
